#include <avr/pgmspace.h>
#include "declarations.h"
#include "arrays.h"

ISR (PCINT1_vect) {
  interruptFlag = true;
  interruptCounter++;
}

void setup() {

  //interrupt init. 
  //(actually don't know the arduino functions for these) 
  PCMSK1 |= (1 << PCINT10);    //pinchange interrupt on PC2
  PCICR |= (1 << PCIE1);       //enable PCINT1 interrupt vector
  PCIFR |= (1 << PCIF1);       //clear interrupt flag for PCIFR1
  DDRC |= (0 << PC2); 

  /* arduino functions are used where execution speed is not a priority
   *  setup and debugging code will use arduino, all timecritical runtime
   *  code was optimized as much as possible for speed, timing functions are 
   *  retained in changeImage   */
  pinMode(highLatch, OUTPUT);
  pinMode(highClock, OUTPUT);
  pinMode(highData, OUTPUT);
  pinMode(lowLatch, OUTPUT);
  pinMode(lowClock, OUTPUT);
  pinMode(lowData, OUTPUT);

  digitalWrite(highLatch, HIGH);
  digitalWrite(lowLatch, HIGH);

  //clear out shift registers before use
  for (int a = 0; a < 16; a++) {
    digitalWrite(lowLatch, LOW);
    digitalWrite(lowData, LOW);
    digitalWrite(lowClock, HIGH);
    digitalWrite(lowClock, LOW);
    digitalWrite(lowLatch, HIGH);
  }
  for (int a = 0; a < 32; a++) {
    digitalWrite(highLatch, LOW);
    digitalWrite(highData, LOW);
    digitalWrite(highClock, HIGH);
    digitalWrite(highClock, LOW);
    digitalWrite(highLatch, HIGH);
  }
  digitalWrite(highLatch, HIGH);
  
  renderBlueArray();
  renderGreenArray();
  renderRedArray();
  Time = micros();
  memcpy(RenderImage, Circle, 110);

  for (int a = 0; a < 12 ; a++) {
    fasterAdvanceLowCLock();
  }
}
byte rowPos = 0;



void loop() {
  if (rowPos > 10) {
    rowPos = 0;
  }
  rowPos++;
  imageToRow(rowPos);
  clearColorArrays();
  fasterAdvanceLowCLock();
  renderBlueArray();
  renderGreenArray();
  renderRedArray();

  changeImage();
  //testArray();
}
void fasterAdvanceLowCLock() {
  PORTB = B00000000;
  if (lowClockCounter == 0) {
    PORTB |= _BV(PB0);
    PORTB |= _BV(PB2);
    PORTB &= ~_BV(PB2);
    lowClockCounter++;
  } else {
    PORTB &= ~_BV(PB0);
    PORTB |= _BV(PB2);
    PORTB &= ~_BV(PB2);

    if (lowClockCounter < 10) {
      lowClockCounter++;
    } else {
      lowClockCounter = 0;
    }
  }
  PORTB |= _BV(PB1);
}

unsigned long recalcAnimationTime(unsigned long lastTime, unsigned long currentTime) {
  return (unsigned long)((currentTime - lastTime) / 10);
}

/*if you suspect a column or row isn't working this is used to figure out
   which it is and speed up finding broken solder joints */
void testArray() {
  for (int a = 0; a < 12; a++) {
    digitalWrite(lowLatch, LOW);
    digitalWrite(lowData, HIGH);
    digitalWrite(lowClock, HIGH);
    delayMicroseconds(writeDelay);
    digitalWrite(lowClock, LOW);
    digitalWrite(lowLatch, HIGH);
    for (int c = 0; c < 20; c++) {
      digitalWrite(highLatch, LOW);
      digitalWrite(highData, LOW);
      digitalWrite(highClock, HIGH);
      delayMicroseconds(writeDelay);
      digitalWrite(highClock, LOW);
      digitalWrite(highLatch, HIGH);
      delay(displayDelay);
    }
    for (int c = 0; c < 10; c++) {
      digitalWrite(highLatch, LOW);
      digitalWrite(highData, HIGH);
      digitalWrite(highClock, HIGH);
      delayMicroseconds(writeDelay);
      digitalWrite(highClock, LOW);
      digitalWrite(highLatch, HIGH);
      delay(20);
    }
  }
}

