/*arrays utilized by the imageToRow() function to make the image compatable with the
 * shift-register operating function renderRows(). */
byte redArray[] = {0, 0, 0, 0, 0, 0, 0 , 0, 0, 0};
byte greenArray[] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
byte blueArray[] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

/* volatile variables utilized in the interrupt routine, these keep the image from 
 *  rotating or otherwise being unviewable.  
 *  optointerrupter connected to atmega328 PC2 utilizes PCINT1_Vect in main file. 
 */
#define interruptPin PC2
volatile boolean interruptFlag = false;
volatile int interruptCounter = 0; 

/*Pin declarations for the board, high pins supplies 5v to the LED matrix columns*/
#define highLatch 3   //ST_CP //PD3
#define highClock 4   //SH_CP //PD4
#define highData 2    //DS    //PD2

/*Pin declarations for the board, low pins connect ground to the LED matrix rows */
#define lowLatch 9    //ST_CP //PB1
#define lowClock 10   //SH_CP //PB2
#define lowData 8     //DS    //PB0

/*images are defined by these values for color in the arrays */
#define RED 1
#define GREEN 2
#define BLUE 3

/* changeImage() variables */
int currentPositionInArrays = 0;           //keeps track of where in 2D array should be displayed
unsigned long Time = 0;                    //used to track how long has passed since a frame has been pushed out 
unsigned frameDisplayTime = 600000;        //how long a frame will be displayed for (10 frames per rotation) constantly adjusted by recalcAnimationTime()
int animationCount = 0;                    //current "image" to be displayed for animations
unsigned long lastInterrupt = 0;           //time since last interrupt 
unsigned long timeOnFrame = 0;             //how long a frame has been displayed for, calculated by (current time - frameUptime)
unsigned long frameUptime = 0;             //used to calculate timeOnFrame stored to everytime a new frame is pushed
/* rotationMarkTime is used to store the time taken for a rotation to be completed, 
 *  compared against frameDisplayTime to determine whether the rotation had been fully completed or not
 *  this value is used to correct the frameDisplayTime in the next rotation through the alignmentAdjust() 
 *  function */
unsigned long rotationMarkTime = 0;

/* utilizied in main loop to track which rows are to be connected to ground at a given time */
int lowClockCounter = 0;

/*debugging only variables */
int writeDelay = 1000;
int displayDelay = 500;

/* primary array utilized to store "image" in ram when pulled from program memory */
byte RenderImage[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                       0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                       0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                       0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                       0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                       0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                       0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                       0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                       0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                       0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                       0, 0, 0, 0, 0, 0, 0, 0, 0, 0
                     };

/* test image pushed to display at the end of the setup (used for debugging) */
const byte Circle[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                        0, 0, 0, 1, 1, 1, 1, 0, 0, 0,
                        0, 0, 1, 0, 0, 0, 0, 1, 0, 0,
                        0, 0, 1, 0, 0, 0, 0, 1, 0, 0,
                        0, 0, 1, 0, 0, 0, 0, 1, 0, 0,
                        0, 0, 1, 0, 0, 0, 0, 1, 0, 0,
                        0, 0, 1, 0, 0, 0, 0, 1, 0, 0,
                        0, 0, 0, 1, 1, 1, 1, 0, 0, 0,
                        0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                        0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                        0, 0, 0, 0, 0, 0, 0, 0, 0, 0
                      };
